#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "process.h"
#include "test.h"
#include "../../process.h"
#include "../../system.h"
#include "../../unittest.h"

static int hw_stop();
static void init_socket();

struct HWStopInterface hw_stop_interface = {
    .hw_stop = hw_stop
};

static void init_socket();

static fd_set sock_fd_set;
static fd_set sock_read_fds;

static struct timeval timeout;

static int server_sock;


void init_hw_stop_interface() {
    printf("Initialization HW Stop Interface\t\t");
    
    set_process(HW_STOP_INTERFACE, &hw_stop_interface);
    SET_UNIT_TEST(HW_STOP_INTERFACE, hw_stop_interface_unit_test);
    
    init_socket();
    printf("\033[92m[OK]\033[0m\n");
}

static int hw_stop() {
    struct sockaddr_in clientaddr;
    int client_len, len;
    char buf[128];
    
    sock_read_fds = sock_fd_set;

    if(select(server_sock+1, &sock_read_fds, 0, 0, &timeout) == -1)
    {
        printf("Server-select() error lol!");
        exit(1);
    }
    
    if(FD_ISSET(server_sock, &sock_read_fds))
    {
        len = recvfrom(server_sock, buf, 128, 0, (struct sockaddr *)&clientaddr, &client_len);
        return buf[0] == 1;
    }
    
    return 0;
}


static void init_socket() {
    struct sockaddr_in serveraddr;
    
    FD_ZERO(&sock_fd_set);
    FD_ZERO(&sock_read_fds);
    
    server_sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (server_sock < 0)
    {
        perror("socket error : ");
        exit(0);
    }
    
    bzero(&serveraddr, sizeof(serveraddr));
    serveraddr.sin_family = AF_INET;
    serveraddr.sin_addr.s_addr = inet_addr("127.0.0.1");
    serveraddr.sin_port = htons(6600);
    
    if(bind(server_sock, (struct sockaddr *)&serveraddr, sizeof(serveraddr)) == -1)
    {
        perror("bind error: ");
        exit(0);
    }
    
    FD_SET(server_sock, &sock_fd_set);
    
    timeout.tv_sec = 0;
    timeout.tv_usec = 0;
}
