#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "../../process.h"
#include "../../unittest.h"
#include "process.h"
#include "test.h"

static struct Packet get_rx_packet();

static void init_server_socket();
int get_server_socket();

static int server_sock = -1;
static int server_socket_max = 0;
static fd_set sock_fd_set;
static fd_set sock_read_fds;

static struct timeval timeout;

// 프로세스 정의
const struct NetworkRxInterface network_rx_interface = {
    .get_rx_packet = get_rx_packet
};

void init_network_rx_interface() {
    printf("Initialization Network Rx Interface...\t\t");
    set_process(NETWORK_RX_INTERFACE, &network_rx_interface);
	SET_UNIT_TEST(NETWORK_RX_INTERFACE, network_rx_interface_unit_test);
	
	init_server_socket();
	printf("\033[92m[OK]\033[0m\n");
}


// 패킷이 온게 있으면 리턴함, 끝까지 안 온거면 무시
static struct Packet get_rx_packet() {
    struct Packet packet;

    int i, a;
    int addrlen;
    
    int new_client;
    struct sockaddr_in clientaddr;
    
    sock_read_fds = sock_fd_set;
    
    packet.data = NULL;
    packet.length = 0;

    if(select(server_socket_max+1, &sock_read_fds, NULL, NULL, &timeout) == -1)
    {
        printf("Server-select() error lol!");
        exit(1);
    }

    for(i = 0; i <= server_socket_max; i++)
    {
        if(FD_ISSET(i, &sock_read_fds))
        {
            if(i == server_sock)
            {
                addrlen = sizeof(clientaddr);
                if((new_client = accept(server_sock, (struct sockaddr *)&clientaddr, &addrlen)) == -1)
                {
                    perror("Server-accept() error lol!");
                }
                
                FD_SET(new_client, &sock_fd_set);

                if(new_client > server_socket_max)
                {
                    server_socket_max = new_client;
                }
                for(a=0; a<64; a++) if(_sock_list[a] == 0) { _sock_list[a] = new_client; break; }
                printf("New connection from %s on socket %d\n", inet_ntoa(clientaddr.sin_addr), new_client);
            } else {
                char header[5];                // version, length
                int result;
                int size;
                
                result = recv(i, header, 5, MSG_WAITALL);
                if(result < 1) {
                    close(i);
                    FD_CLR(i, &sock_fd_set);
                    
                    for(a=0; a<64; a++) if(_sock_list[a] == i) { _sock_list[a] = 0; }
                    continue;
                }
                
                // 빅 엔디안 사이즈 계산
                size = ((header[1] << 24)&0xff000000) | ((header[2] << 16)&0xff0000) | ((header[3] << 8)&0xff00) | (header[4]&0xff);
                
                char * buf = (char *)malloc(sizeof(char) * size);
                result = recv(i, buf, size, MSG_WAITALL);
                if(result < 1) {
                    close(i);
                    FD_CLR(i, &sock_fd_set);
                    
                    for(a=0; a<64; a++) if(_sock_list[a] == i) { _sock_list[a] = 0; }
                } else if(size == result) {
                    packet.data = buf;
                    packet.length = size;
                    packet.sock = i;
                    return packet;
                } else {
                    printf("WTF!!!!!!!!!!!!!!!! RX!!!!!\n");
                }
                
                free(buf);
            }
        }
    }

    return packet;
}


static void init_server_socket() {
    struct sockaddr_in serveraddr;
    int i;
    int yes = 1;
    
    FD_ZERO(&sock_fd_set);
    FD_ZERO(&sock_read_fds);
    
    server_sock = socket(AF_INET, SOCK_STREAM, 0);
    if(server_sock == -1) {
        printf("SOCKET INIT FAIL.");
        exit(1);
    }

    if(setsockopt(server_sock, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1)
    {
        perror("setsockopt FAIL.");
        exit(1);
    }

    serveraddr.sin_family = AF_INET;
    serveraddr.sin_addr.s_addr = INADDR_ANY;
    serveraddr.sin_port = htons(6000);
    memset(&(serveraddr.sin_zero), 0, 8);
    if(bind(server_sock, (struct sockaddr *)&serveraddr, sizeof(serveraddr)) == -1)
    {
        perror("Server-bind() error lol!");
        exit(1);
    }

    if(listen(server_sock, 10) == -1)
    {
         perror("Server-listen() error lol!");
         exit(1);
    }

    FD_SET(server_sock, &sock_fd_set);
    server_socket_max = server_sock;
    
    timeout.tv_sec = 0;
    timeout.tv_usec = 0;
    
    for(i=0; i<64; i++) _sock_list[i]=0;
}

int get_server_socket() {
    return server_sock;
}