#include <stdlib.h>
#include "process.h"
#include "test.h"
#include "../../../../process.h"
#include "../../../../system.h"
#include "../../../../unittest.h"
#include "../../../../databases/user_database/process.h"
#include "../fail_auth_dispatcher/process.h"
#include "../../../../interfaces/network_rx_interface/process.h"
#include "../print_validation_process/process.h"
#include "../request_to_cancel_printing_process/process.h"
#include "../user_creation_process/process.h"
#include "../user_removal_process/process.h"
#include "../user_listing_process/process.h"

static void tick();

const struct AuthAndDispatchController auth_and_dispatch_controller = {
    .tick = tick
};

static void (*current_state)();
static void ready();

void init_auth_and_dispatch_controller() {
    printf("Initialization Auth&Dispath Controller...\t\t");
    
    set_process(AUTH_AND_DISPATCH_CONTROLLER, &auth_and_dispatch_controller);
    SET_UNIT_TEST(AUTH_AND_DISPATCH_CONTROLLER, auth_and_dispatch_controller_unit_test);
    add_tick_listener(auth_and_dispatch_controller.tick);
    
    printf("\033[92m[OK]\033[0m\n");
    
    current_state = ready;
}

static void tick() {
    current_state();
}


static struct UserInfo *get_user(char *user_id, char *user_pw) { //0~11: user_id, 12~23:user_pw
    int i;
    int fail = 0;
    struct UserDatabase *db = get_process(USER_DATABASE);
    struct UserInfo* user = NULL;
    
    
    for(user=db->header->nextInfo; user != db->tailer; user=user->nextInfo) {
        fail = 0;
        for(i=0; i<USER_ID_MAX_LENGTH && !fail; i++) {
            if(user->user_id[i] != user_id[i]) fail=1;
        }
        for(i=0; i<USER_PW_MAX_LENGTH && !fail; i++) {
            if(user->user_pw[i] != user_pw[i]) fail=1;
        }
        if(!fail) return user;
    }
    
    return 0;
}

static void fail_auth() { current_state = ready; }
static void request_to_print() { current_state = ready; }
static void request_to_cancel_printing() { current_state = ready; }
static void user_creation() { current_state = ready; }
static void user_removal() { current_state = ready; }
static void user_listing() { current_state = ready; }


static void ready() {
    struct Packet packet;
    struct UserInfo *user;
    struct NetworkRxInterface *network = get_process(NETWORK_RX_INTERFACE);
    
    packet = network->get_rx_packet();
    if(packet.data == NULL) return;
    
    /*
    char req_type;
    int req_id;
    char user_id[12];
    char user_pw[12];
    // Payload ~~
    */
    
    if(packet.length < TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH) return;

    user = get_user(packet.data + TYPE_LENGTH + REQ_ID_LENGTH, packet.data + TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH);

    if(!user) {
        ((struct FailAuthDispatcher *)get_process(FAIL_AUTH_DISPATCHER))->trigger(&packet);
        current_state = fail_auth;
        return;
    }
    
    char req_type = packet.data[0];
    if(req_type == PACKET_TYPE_REQUEST) { // 인쇄 요청
        ((struct PrintValidationProcess *)get_process(PRINT_VALIDATION_PROCESS))->trigger(&packet);
        current_state = request_to_print;
        return;
    } else if(req_type == PACKET_TYPE_CANCEL) { // 인쇄 취소
        ((struct RequestToCancelPrintingProcess *)get_process(REQUEST_TO_CANCEL_PRINTING_PROCESS))->trigger(&packet, user->is_admin);
        current_state = request_to_cancel_printing;
        return;
    } else { // 관리자 명령어
        if(!user->is_admin) { // 권한 체크
            ((struct FailAuthDispatcher *)get_process(FAIL_AUTH_DISPATCHER))->trigger(&packet);
            current_state = fail_auth;
            return;
        }
        
        if(req_type == PACKET_TYPE_USER_CREATION) {
            ((struct UserCreationProcess *)get_process(USER_CREATION_PROCESS))->trigger(&packet);
            current_state = user_creation;
            return;
        } else if(req_type == PACKET_TYPE_USER_REMOVAL) {
            ((struct UserRemovalProcess *)get_process(USER_REMOVAL_PROCESS))->trigger(&packet);
            current_state = user_removal;
            return;
        } else if(req_type == PACKET_TYPE_USER_LISTING) {
            ((struct UserListingProcess *)get_process(USER_LISTING_PROCESS))->trigger(&packet);
            current_state = user_listing;
            return;
        }
    }
}
