#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../../databases/user_database/process.h"
#include "../../../../interfaces/network_rx_interface/process.h"
#include "../fail_auth_dispatcher/process.h"
#include "../print_validation_process/process.h"
#include "../request_to_cancel_printing_process/process.h"
#include "../user_creation_process/process.h"
#include "../user_removal_process/process.h"
#include "../user_listing_process/process.h"


static struct UserInfo v_header;
static struct UserInfo v_tailer;

static char test_packet_data[29];
static struct Packet test_packet;

static struct Packet test_get_rx_packet(){
    return test_packet;
};

static struct NetworkRxInterface test_network_rx_interface = {
    .get_rx_packet = test_get_rx_packet
};

static struct UserDatabase test_user_database = {
    .header = &v_header,
    .tailer = &v_tailer
};

static int test_trigger_id = -1;

#define ADD_TEST_TRIGGER(STRUCT_NAME, VAR_NAME, VALUE) \
    static void VAR_NAME##_trigger(struct Packet packet) { \
        test_trigger_id = VALUE; \
    } \
    static struct STRUCT_NAME VAR_NAME = { \
        .trigger = VAR_NAME##_trigger \
    };


ADD_TEST_TRIGGER(FailAuthDispatcher, test_fad, 0)
ADD_TEST_TRIGGER(PrintValidationProcess, test_pvp, 1)
ADD_TEST_TRIGGER(RequestToCancelPrintingProcess, test_rtcpp, 2)
ADD_TEST_TRIGGER(UserCreationProcess, test_ucp, 3)
ADD_TEST_TRIGGER(UserRemovalProcess, test_urp, 4)
ADD_TEST_TRIGGER(UserListingProcess, test_ulp, 5)


static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();
static void unit_test_6();
static void unit_test_7();


void auth_and_dispatch_controller_unit_test() {
    struct UserInfo *test_user;
    // 의존성 있는 프로세스 정의 (프로세스에서 호출하는 것들)
    set_process(USER_DATABASE, &test_user_database);
    set_process(NETWORK_RX_INTERFACE, &test_network_rx_interface);
    set_process(FAIL_AUTH_DISPATCHER, &test_fad);
    set_process(PRINT_VALIDATION_PROCESS, &test_pvp);
    set_process(REQUEST_TO_CANCEL_PRINTING_PROCESS, &test_rtcpp);
    set_process(USER_CREATION_PROCESS, &test_ucp);
    set_process(USER_REMOVAL_PROCESS, &test_urp);
    set_process(USER_LISTING_PROCESS, &test_ulp);
    
    // 테스트용 유저 생성
    // 관리자 계정 하나, 일반 사용자 하나 
    test_user = (struct UserInfo *)malloc(sizeof(struct UserInfo));
    memset(test_user->user_id, 0, 12);
    memset(test_user->user_pw, 0, 12);
    strcpy(test_user->user_id, "admin");
    strcpy(test_user->user_pw, "admin");
    test_user->is_admin = 1;
    
    v_header.nextInfo = test_user;
    test_user->prevInfo = &v_header;
    test_user->nextInfo = &v_tailer;
    v_tailer.prevInfo = test_user;
    
    test_user = (struct UserInfo *)malloc(sizeof(struct UserInfo));
    memset(test_user->user_id, 0, 12);
    memset(test_user->user_pw, 0, 12);
    strcpy(test_user->user_id, "user0");
    strcpy(test_user->user_pw, "pass0");
    test_user->is_admin = 0;
    
    v_header.nextInfo->nextInfo = test_user;
    test_user->prevInfo = v_header.nextInfo;
    test_user->nextInfo = &v_tailer;
    v_tailer.prevInfo = test_user;
    
    test_packet.sock = 99;
    test_packet.length = 29;
    test_packet.data = test_packet_data;
    memset(test_packet_data, 0, 29);
    
    // 테스트
    unit_test_1();
    unit_test_2();
    unit_test_3();
    unit_test_4();
    unit_test_5();
    unit_test_6();
    unit_test_7();
    
    free(v_header.nextInfo);
    free(v_tailer.prevInfo);
}

// 현재 상태가 대기이며 인쇄 요청 패킷이옴 (권한 실패)
static void unit_test_1() {
    struct AuthAndDispatchController *controller = get_process(AUTH_AND_DISPATCH_CONTROLLER);

    test_packet_data[0] = 0;
    controller->tick();
    controller->tick();
    controller->tick();
    
    unit_test_assert("NPS UTC 3000_000 AUTH_AND_DISPATCH_CONTROLLER - unit_test_1", test_trigger_id == 0);
}

// 관리자 명령이 온 경우
static void unit_test_2() {
    struct AuthAndDispatchController *controller = get_process(AUTH_AND_DISPATCH_CONTROLLER);

    test_packet_data[0] = 2;
    strcpy(test_packet_data + 5, "user0");
    strcpy(test_packet_data + 5 + 12, "pass0");
    controller->tick();
    controller->tick();
    controller->tick();

    unit_test_assert("NPS UTC 3000_001 AUTH_AND_DISPATCH_CONTROLLER - unit_test_2", test_trigger_id == 0);
}


// 
static void unit_test_3() {
    struct AuthAndDispatchController *controller = get_process(AUTH_AND_DISPATCH_CONTROLLER);
    
    test_packet_data[0] = 0;
    strcpy(test_packet_data + 5, "admin");
    strcpy(test_packet_data + 5 + 12, "admin");
    
    controller->tick();
    controller->tick();
    controller->tick();

    unit_test_assert("NPS UTC 3000_002 AUTH_AND_DISPATCH_CONTROLLER - unit_test_3", test_trigger_id == 1);
}


static void unit_test_4() {
    struct AuthAndDispatchController *controller = get_process(AUTH_AND_DISPATCH_CONTROLLER);
    
    test_packet_data[0] = 1;
    
    controller->tick();
    controller->tick();
    controller->tick();

    unit_test_assert("NPS UTC 3000_003 AUTH_AND_DISPATCH_CONTROLLER - unit_test_4", test_trigger_id == 2);
}

static void unit_test_5() {
    struct AuthAndDispatchController *controller = get_process(AUTH_AND_DISPATCH_CONTROLLER);
    
    test_packet_data[0] = 2;
    
    controller->tick();
    controller->tick();
    controller->tick();

    unit_test_assert("NPS UTC 3000_004 AUTH_AND_DISPATCH_CONTROLLER - unit_test_5", test_trigger_id == 3);
}

static void unit_test_6() {
    struct AuthAndDispatchController *controller = get_process(AUTH_AND_DISPATCH_CONTROLLER);
    
    test_packet_data[0] = 3;
    
    controller->tick();
    controller->tick();
    controller->tick();

    unit_test_assert("NPS UTC 3000_005 AUTH_AND_DISPATCH_CONTROLLER - unit_test_6", test_trigger_id == 4);
}

static void unit_test_7() {
    struct AuthAndDispatchController *controller = get_process(AUTH_AND_DISPATCH_CONTROLLER);
    
    test_packet_data[0] = 4;
    
    controller->tick();
    controller->tick();
    controller->tick();

    unit_test_assert("NPS UTC 3000_006 AUTH_AND_DISPATCH_CONTROLLER - unit_test_7", test_trigger_id == 5);
}
