#include <stdlib.h>
#include <string.h>
#include "process.h"
#include "test.h"
#include "../../../../process.h"
#include "../../../../system.h"
#include "../../../../unittest.h"
#include "../../../../databases/job_queue/process.h"
#include "../../../../databases/ink_paper_state_database/process.h"
#include "../../../../interfaces/network_rx_interface/process.h"
#include "../../../../interfaces/network_tx_interface/process.h"

static struct JobInfo test_jobs_info[JOB_MAX_SIZE];

static int current_ink_value = 0;
static int current_paper_value = 0;
static int wait_refill_ink_value = 0;
static int wait_refill_paper_value = 0;

static struct InkPaperStateDatabase test_ink_paper_state_database = {
    .current_ink_value = &current_ink_value,
    .current_paper_value = &current_paper_value,
    .wait_refill_ink_value = &wait_refill_ink_value,
    .wait_refill_paper_value = &wait_refill_paper_value
    
};
static struct JobQueue test_job_queue = {
    .jobs_info = &test_jobs_info
};

static int flag = 0;

static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();
static void unit_test_6();
static void unit_test_7();
static void unit_test_8();
static void test_send_packet(struct Packet);

static struct NetworkTxInterface test_network_tx_interface = {
    .send_packet = test_send_packet
};

void print_validation_process_unit_test() {
    int i;
    set_process(JOB_QUEUE, &test_job_queue);
    set_process(INK_PAPER_STATE_DATABASE, &test_ink_paper_state_database);
    set_process(NETWORK_TX_INTERFACE, &test_network_tx_interface);
    
    // Test Job Queue 생성 : 모두 채우기
    //
    for ( i = 0 ; i < JOB_MAX_SIZE; i++ )
    {
        memset(&test_jobs_info[i], 0, sizeof(struct JobInfo));
    }

    unit_test_1();
    
    for ( i = 0 ; i < JOB_MAX_SIZE ; i++ ) test_jobs_info[i].job_id = -1;
    
    unit_test_2();
    unit_test_3();
    unit_test_4();
    unit_test_5();
    unit_test_6();
    unit_test_7();
    unit_test_8();
}

// Test for get_empty_job_index
static void unit_test_1() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    struct JobQueue *jq = get_process(JOB_QUEUE);
    int res;
    
    test_jobs_info[JOB_MAX_SIZE-2].job_id = -1;
    
    res = process->_get_empty_job_index();
    unit_test_assert("NPS UTC 3000_004 PRINT_VALIDATION_PROCESS - unit_test_1", 
        ( res == (JOB_MAX_SIZE - 2) )
    );
}

// Test for get_require_ink
static void unit_test_2() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    // ex) 한 줄에 40글자와 공백이 5개(30글자내에) 있을 경우 잉크소모는 25(30-5)이다.
    int res;
    char *buf = "Cowabunga Cowabunga Cowabunga \n";
    
    // 공백 3개, 글자 27개이므로 잉크소모는 27만큼 들것임.
    res = process->_get_require_ink(buf, strlen(buf));
    unit_test_assert("NPS UTC 3000_005 PRINT_VALIDATION_PROCESS - unit_test_2", 
        ( res == 27 )
    );
}

// Test for get_require_paper
static void unit_test_3() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    // 20줄
    char *buf = "a\na\na\na\na\na\na\na\na\na\na\na\na\na\na\na\na\na\na\na\n";
    int size = strlen(buf);
    int res;
    
    res = process->_get_require_paper(buf, size);
    
    unit_test_assert("NPS UTC 3000_006 PRINT_VALIDATION_PROCESS - unit_test_3", 
        (res == 2)
    );
}

// Test for get_job_queue_require_ink
static void unit_test_4() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    int res;
    
    res = process->_get_job_queue_require_ink();
    
    unit_test_assert("NPS UTC 3000_007 PRINT_VALIDATION_PROCESS - unit_test_4", 
        (res == 0)
    );
}

// Test for get_job_queue_require_paper
static void unit_test_5() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    int res;
    
    res = process->_get_job_queue_require_paper();
    
    unit_test_assert("NPS UTC 3000_008 PRINT_VALIDATION_PROCESS - unit_test_5", 
        (res == 0)
    );
}

// Test for trigger 
// 
static void unit_test_6() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    char test_packet_data[29];
    struct Packet test_packet = {
        .data = test_packet_data,
        .length = 29
    };
    
    test_packet_data[0] = PACKET_TYPE_REQUEST;
    
    process->trigger(&test_packet);
    
    printf("%d\n", flag);
    
    unit_test_assert("NPS UTC 3000_009 PRINT_VALIDATION_PROCESS - unit_test_6", flag == 0 );
}

// lack of ink
static void unit_test_7() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    char test_packet_data[34];
    struct Packet test_packet = {
        .data = test_packet_data,
        .length = 34
    };
    
    current_ink_value = 0;
    current_paper_value = 1;
    
    memset(test_packet_data, 0, 34);
    test_packet_data[0] = PACKET_TYPE_REQUEST;
    test_packet_data[32] = 1;
    test_packet_data[33] = 'A';
    
    process->trigger(&test_packet);
    
    unit_test_assert("NPS UTC 3000_010 PRINT_VALIDATION_PROCESS - unit_test_7", flag == 2 );
}

// lack of paper
static void unit_test_8() {
    struct PrintValidationProcess *process = get_process(PRINT_VALIDATION_PROCESS);
    char test_packet_data[34];
    
    struct Packet test_packet = {
        .data = test_packet_data,
        .length = 34
    };
    
    current_ink_value = 1;
    current_paper_value = 0;
    
    memset(test_packet_data, 0, 34);
    test_packet_data[0] = PACKET_TYPE_REQUEST;
    test_packet_data[32] = 1;
    test_packet_data[33] = 'A';
    
    process->trigger(&test_packet);
    
    unit_test_assert("NPS UTC 3000_011 PRINT_VALIDATION_PROCESS - unit_test_8", flag == 3 );
}

static void test_send_packet(struct Packet p) {
    // flag will get res_type 
    flag = p.data[5];
}