#include "process.h"
#include "test.h"
#include "../../../../process.h"
#include "../../../../system.h"
#include "../../../../unittest.h"
#include "../../../../interfaces/network_rx_interface/process.h"
#include "../../../../databases/job_queue/process.h"
#include "../../../../databases/user_database/process.h"

static void trigger();

static struct RequestToCancelPrintingProcess request_to_cancel_printing_process = {
    .trigger = trigger
};


void init_request_to_cancel_printing_process() {
    printf("Initialization Request to Cancel Printing Dispatcher...\t\t");
    set_process(REQUEST_TO_CANCEL_PRINTING_PROCESS, &request_to_cancel_printing_process);
    SET_UNIT_TEST(REQUEST_TO_CANCEL_PRINTING_PROCESS, request_to_cancel_printing_process_unit_test);
    printf("\033[92m[OK]\033[0m\n");
}


static int get_job_info_index(int id) {
    struct JobQueue *job_queue = get_process(JOB_QUEUE);
    
    int i;
    for(i = 0; i < JOB_MAX_SIZE; i++){
        if(job_queue->jobs_info[i].job_id == id) return i;
    }
    return -1;
}

static int user_id_cmp(char *a, char *b) {
    int i;
    for(i=0; i<USER_ID_MAX_LENGTH; i++) if(a[i] != b[i]) return 1;
    return 0;
}


static void trigger(struct Packet *packet, int is_admin) {
    int i;
    int index;
    struct JobQueue *job_queue = get_process(JOB_QUEUE);
    
    if(packet->length < TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + JOB_ID_LENGTH) return;
    
    char* job_id = packet->data + TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH;

    index = get_job_info_index(
        ((job_id[0] << 24) & 0xff000000) |
        ((job_id[1] << 16) & 0xff0000) |
        ((job_id[2] << 8) & 0xff00) |
        ((job_id[3]) & 0xff)
    );
    
    if(index == -1) return;
    
    if(!is_admin && user_id_cmp(job_queue->jobs_info[index].user_id, packet->data + TYPE_LENGTH + REQ_ID_LENGTH)) return;
    
    if(index == 0) {
        job_queue->jobs_info[index].state = REMOVING;
    } else {
        for(i=index+1; i < JOB_MAX_SIZE; i++) {
            job_queue->jobs_info[i - 1] = job_queue->jobs_info[i];
        }
        
        job_queue->jobs_info[JOB_MAX_SIZE-1].fd = 0;
        job_queue->jobs_info[JOB_MAX_SIZE-1].job_id = -1;
        job_queue->jobs_info[JOB_MAX_SIZE-1].total_page = 0;
    }
}