#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "process.h"
#include "test.h"
#include "../../../../process.h"
#include "../../../../system.h"
#include "../../../../unittest.h"
#include "../../../../databases/job_queue/process.h"
#include "../../../../databases/user_database/process.h"

static struct JobInfo test_jobs_info[JOB_MAX_SIZE];

static struct JobQueue test_job_queue = {
    .jobs_info = test_jobs_info
};

static struct UserInfo v_header;
static struct UserInfo v_tailer;

static struct UserDatabase test_user_database = {
    .header = &v_header,
    .tailer = &v_tailer
};

static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();

static void init_test_job_queue() {
    int i;
    
    // Test Job Queue 생성,  작업은 모두 차있다.
    for ( i = 0 ; i < JOB_MAX_SIZE ; i++ ) 
    {
        test_jobs_info[i].fd = NULL;
        test_jobs_info[i].job_id = i+1;
        memset(test_jobs_info[i].user_id, 0, 12);
        sprintf(test_jobs_info[i].user_id, "user%d", i);
        test_jobs_info[i].current_page = rand() % 6;
        test_jobs_info[i].total_page = i+5;
        test_jobs_info[i].state = 0;
        test_jobs_info[i].buffer = (char*)malloc(32);
        memset(test_jobs_info[i].buffer, 65+i, 31);
        test_jobs_info[i].buffer[31] = 0;
    }
    
}

void request_to_cancel_printing_process_unit_test() {
    int i;
    struct UserInfo *test_user;
    
    set_process(JOB_QUEUE, &test_job_queue);
    set_process(USER_DATABASE, &test_user_database);
    
    test_user = (struct UserInfo *)malloc(sizeof(struct UserInfo));
    memset(test_user->user_id, 0, 12);
    memset(test_user->user_pw, 0, 12);
    strcpy(test_user->user_id, "admin");
    strcpy(test_user->user_pw, "admin");
    test_user->is_admin = 1;
    
    v_header.nextInfo = test_user;
    test_user->prevInfo = &v_header;
    test_user->nextInfo = &v_tailer;
    v_tailer.prevInfo = test_user;
    
    for ( i = 0 ; i < 5 ; i++ )
    {
        test_user = (struct UserInfo *)malloc(sizeof(struct UserInfo));
        memset(test_user, 0, sizeof(struct UserInfo));
        sprintf(test_user->user_id, "user%d", i);
        sprintf(test_user->user_pw, "pass%d", i);
        test_user->is_admin = 0;
        
        v_tailer.prevInfo->nextInfo = test_user;
        test_user->prevInfo = v_tailer.prevInfo;
        test_user->nextInfo = &v_tailer;
        v_tailer.prevInfo = test_user;
    }
    init_test_job_queue();
    
    unit_test_1();
    unit_test_2();
    unit_test_3();
    unit_test_4();
    unit_test_5();
    
    test_user = v_header.nextInfo->nextInfo;
    
    while ( test_user->nextInfo ) 
    {
        free(test_user->prevInfo);
        test_user = test_user->nextInfo;
    }
}


// 첫번째 작업 취소
static void unit_test_1() {
    struct RequestToCancelPrintingProcess *process = get_process(REQUEST_TO_CANCEL_PRINTING_PROCESS);
    struct Packet test_packet = {
       .data = "\0\0\0\0\0user0\0\0\0\0\0\0\0pass0\0\0\0\0\0\0\0\0\0\0\1",
       .length = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + 4,
       .sock = 0
    };
    
    init_test_job_queue();
    
    process->trigger(&test_packet, 0);

    unit_test_assert("NPS UTC 3000_011 REQUEST_TO_CANCEL_PRINTING_PROCESS - unit_test_1", test_jobs_info[0].state == REMOVING);
}

// 두번째 작업 취소
static void unit_test_2() {
    struct RequestToCancelPrintingProcess *process = get_process(REQUEST_TO_CANCEL_PRINTING_PROCESS);
    struct Packet test_packet = {
       .data = "\0\0\0\0\0user1\0\0\0\0\0\0\0pass0\0\0\0\0\0\0\0\0\0\0\2",
       .length = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + 4,
       .sock = 0
    };
    
    init_test_job_queue();
    
    process->trigger(&test_packet, 0);
    
    unit_test_assert("NPS UTC 3000_012 REQUEST_TO_CANCEL_PRINTING_PROCESS - unit_test_2",
    test_jobs_info[0].job_id == 1 &&
    test_jobs_info[1].job_id == 3 &&
    test_jobs_info[2].job_id == 4 &&
    test_jobs_info[3].job_id == 5 &&
    test_jobs_info[4].job_id == -1
    );
}

// 세번째 작업 취소
static void unit_test_3() {
    struct RequestToCancelPrintingProcess *process = get_process(REQUEST_TO_CANCEL_PRINTING_PROCESS);
    struct Packet test_packet = {
       .data = "\0\0\0\0\0user2\0\0\0\0\0\0\0pass2\0\0\0\0\0\0\0\0\0\0\3",
       .length = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + 4,
       .sock = 0
    };
    
    init_test_job_queue();
    
    process->trigger(&test_packet, 0);
    
    unit_test_assert("NPS UTC 3000_013 REQUEST_TO_CANCEL_PRINTING_PROCESS - unit_test_3",
    test_jobs_info[0].job_id == 1 &&
    test_jobs_info[1].job_id == 2 &&
    test_jobs_info[2].job_id == 4 &&
    test_jobs_info[3].job_id == 5 &&
    test_jobs_info[4].job_id == -1
    );
}


// 네번째 작업 취소
static void unit_test_4() {
    struct RequestToCancelPrintingProcess *process = get_process(REQUEST_TO_CANCEL_PRINTING_PROCESS);
    struct Packet test_packet = {
       .data = "\0\0\0\0\0user3\0\0\0\0\0\0\0pass3\0\0\0\0\0\0\0\0\0\0\4",
       .length = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + 4,
       .sock = 0
    };
    
    init_test_job_queue();
    
    process->trigger(&test_packet, 0);
    
    unit_test_assert("NPS UTC 3000_014 REQUEST_TO_CANCEL_PRINTING_PROCESS - unit_test_4",   
    test_jobs_info[0].job_id == 1 &&
    test_jobs_info[1].job_id == 2 &&
    test_jobs_info[2].job_id == 3 &&
    test_jobs_info[3].job_id == 5 &&
    test_jobs_info[4].job_id == -1
    );
}


// 다섯번째 작업 취소
static void unit_test_5() {
    struct RequestToCancelPrintingProcess *process = get_process(REQUEST_TO_CANCEL_PRINTING_PROCESS);
    struct Packet test_packet = {
       .data = "\0\0\0\0\0user4\0\0\0\0\0\0\0pass4\0\0\0\0\0\0\0\0\0\0\5",
       .length = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + 4,
       .sock = 0
    };
    
    init_test_job_queue();
    
    process->trigger(&test_packet, 0);
    
    unit_test_assert("NPS UTC 3000_015 REQUEST_TO_CANCEL_PRINTING_PROCESS - unit_test_5",   
    test_jobs_info[0].job_id == 1 &&
    test_jobs_info[1].job_id == 2 &&
    test_jobs_info[2].job_id == 3 &&
    test_jobs_info[3].job_id == 4 &&
    test_jobs_info[4].job_id == -1
    );
}

// 관리자 계정으로 작업 취소