#include <string.h>
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../../databases/user_database/process.h"
#include "../../../../interfaces/network_tx_interface/process.h"

static int exist_user(char *);
static void trigger(struct Packet *);

// 프로세스 정의
static struct UserCreationProcess user_creation_process = {
    .trigger = trigger,
    ._exist_user = exist_user
};


void init_user_creation_process() {
    printf("Initialization User Creation Process...\t\t");
    set_process(USER_CREATION_PROCESS, &user_creation_process);
	SET_UNIT_TEST(USER_CREATION_PROCESS, user_creation_process_unit_test);
	printf("\033[92m[OK]\033[0m\n");
}

static int exist_user(char *user_id) {
    int i;
    int flag = 0;
    struct UserDatabase *db = get_process(USER_DATABASE);
    struct UserInfo *user;

    for(user=db->header->nextInfo; user != db->tailer; user=user->nextInfo) {
        flag = 0;
        
        for(i=0; i<USER_ID_MAX_LENGTH && !flag; i++) {
            if(user->user_id[i] != user_id[i]) {
                flag = 1;
                break;
            }
        }
        
        if(!flag) return 1;
    }
    
    return 0;
}

static void trigger(struct Packet *packet)
{
    struct Packet res;
    char res_data[TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH];
    memcpy(res_data, packet->data, TYPE_LENGTH + REQ_ID_LENGTH);
    
    res.data = res_data;
    res.length = TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH;
    res.sock = packet->sock;
    
    
    const int start = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH;
    if(packet->length < start + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH) return;
    
    char *target_id = packet->data + start;
    char *target_pw = packet->data + start + USER_ID_MAX_LENGTH;
    
    // exist id
    if(exist_user(target_id)) {
        res_data[TYPE_LENGTH + REQ_ID_LENGTH] = 1;
        ((struct NetworkTxInterface *)get_process(NETWORK_TX_INTERFACE))->send_packet(res);
        return;
    }
    
    
    struct UserDatabase *db = get_process(USER_DATABASE);
    struct UserInfo *tmp;
    
    // 유저 정보 생성
    struct UserInfo *user_creation_request = (struct UserInfo*)malloc(sizeof(struct UserInfo));
    memcpy(user_creation_request->user_id, target_id, USER_ID_MAX_LENGTH);
    memcpy(user_creation_request->user_pw, target_pw, USER_PW_MAX_LENGTH);
    user_creation_request->is_admin = 0;

    tmp = db->tailer->prevInfo;
    
    // 유저 리스트에 연결 생성 
    tmp->nextInfo = user_creation_request;
    user_creation_request->prevInfo = tmp;
    user_creation_request->nextInfo = db->tailer;
    db->tailer->prevInfo = user_creation_request;
    
    res_data[TYPE_LENGTH + REQ_ID_LENGTH] = 0;
    
    
    ((struct NetworkTxInterface *)get_process(NETWORK_TX_INTERFACE))->send_packet(res);

    /*
        const int start = TYPE_LENGTH + REQ_ID_LENGTH;
    struct InkPaperStateDatabase *ink_paper_db = get_process(INK_PAPER_STATE_DATABASE);
    struct JobQueue *job_queue = get_process(JOB_QUEUE);
    int index;
    struct Packet res;
    char res_data[TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH + JOB_ID_LENGTH];
    memcpy(res_data, packet->data, TYPE_LENGTH + REQ_ID_LENGTH);
    
    res.data = res_data;
    res.length = TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH + JOB_ID_LENGTH;
    res.sock = packet->sock;
    
    index = get_empty_job_index();
    */
}
