#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../../databases/user_database/process.h"
#include "../../../../interfaces/network_tx_interface/process.h"

static int a = 0;

static struct UserInfo v_header;
static struct UserInfo v_tailer;

static void test_send_packet();

static struct UserDatabase test_user_database = {
    .header = &v_header,
    .tailer = &v_tailer
};


static struct NetworkTxInterface test_tx_interface = {
    .send_packet = test_send_packet
};

static void unit_test__exist_user_1();
static void unit_test__exist_user_2();
static void unit_test__exist_user_3();
static void unit_test__exist_user_4();
static void unit_test__exist_user_5();
static void unit_test_1();
static void unit_test_2();


void user_creation_process_unit_test() {
    // 의존성 있는 프로세스 정의 (프로세스에서 호출하는 것들)
    set_process(USER_DATABASE, &test_user_database);
    set_process(NETWORK_TX_INTERFACE, &test_tx_interface);
    
    // 테스트를 위한 유저 데이터베이스 초기화
    v_header.prevInfo = NULL;
    v_header.nextInfo = &v_tailer;
    v_tailer.prevInfo = &v_header;
    v_tailer.nextInfo = NULL;
    // 테스트
    unit_test__exist_user_1();
    unit_test__exist_user_2();
    unit_test__exist_user_3();
    unit_test__exist_user_4();
    unit_test_1();
    unit_test_2();
}

// _exist_user 함수가 유저를 정상적으로 찾아내는가
static void unit_test__exist_user_1(){
    struct UserCreationProcess *process = get_process(USER_CREATION_PROCESS);
    struct UserInfo test_user = {
        .user_id = "test\0\0\0\0\0\0\0\0"
    };
    
    v_header.nextInfo = &test_user;
    test_user.prevInfo = &v_header;
    test_user.nextInfo = &v_tailer;
    v_tailer.prevInfo = &test_user;
    
    unit_test_assert("NPS UTC 3000_016 USER_CREATION_PROCESS - unit_test__exist_user_1", process->_exist_user("test\0\0\0\0\0\0\0\0"));
}

//DB에 유저 정보가 없을경우 _exist_user 함수가 유저가 없다고 제대로 체크하는가
static void unit_test__exist_user_2(){
    struct UserCreationProcess *process = get_process(USER_CREATION_PROCESS);
    v_header.prevInfo = NULL;
    v_header.nextInfo = &v_tailer;
    v_tailer.prevInfo = &v_header;
    v_tailer.nextInfo = NULL;
    unit_test_assert("NPS UTC 3000_017 USER_CREATION_PROCESS - unit_test__exist_user_2", !process->_exist_user("test\0\0\0\0\0\0\0\0"));
}


//DB에 유저 정보가 있을경우 잘 찾는지(1)

static void unit_test__exist_user_3(){
    struct UserCreationProcess *process = get_process(USER_CREATION_PROCESS);
    struct UserInfo test_user;
    struct UserInfo test_user2;
    struct UserInfo test_user3;
    
    memset(test_user.user_id, 0, 12);
    memset(test_user2.user_id, 0, 12);
    memset(test_user3.user_id, 0, 12);
    strcpy(test_user.user_id, "test");
    strcpy(test_user2.user_id, "a");
    strcpy(test_user3.user_id, "b");
    
    v_header.nextInfo = &test_user;
    
    test_user.prevInfo = &v_header;
    test_user.nextInfo = &test_user2;
    
    test_user2.prevInfo = &test_user;
    test_user2.nextInfo = &test_user3;
    
    test_user3.prevInfo = &test_user2;
    test_user3.nextInfo = &v_tailer;
    
    v_tailer.prevInfo = &test_user3;
    
    unit_test_assert("NPS UTC 3000_018 USER_CREATION_PROCESS - unit_test__exist_user_3", process->_exist_user("test\0\0\0\0\0\0\0\0"));
}

//DB에 유저 정보가 있을경우 잘 찾는지(2)

static void unit_test__exist_user_4(){
    struct UserCreationProcess *process = get_process(USER_CREATION_PROCESS);
    struct UserInfo test_user;
    struct UserInfo test_user2;
    struct UserInfo test_user3;
    
    memset(test_user.user_id, 0, 12);
    memset(test_user2.user_id, 0, 12);
    memset(test_user3.user_id, 0, 12);
    strcpy(test_user.user_id, "test");
    strcpy(test_user2.user_id, "a");
    strcpy(test_user3.user_id, "b");
    
    v_header.nextInfo = &test_user;
    
    test_user2.prevInfo = &v_header;
    test_user2.nextInfo = &test_user;
    
    test_user.prevInfo = &test_user2;
    test_user.nextInfo = &test_user3;
    
    test_user3.prevInfo = &test_user;
    test_user3.nextInfo = &v_tailer;
    
    v_tailer.prevInfo = &test_user3;
    
    unit_test_assert("NPS UTC 3000_019 USER_CREATION_PROCESS - unit_test__exist_user_4", process->_exist_user("test\0\0\0\0\0\0\0\0"));

}
//DB에 유저 정보가 있을경우 잘 찾는지(3)

static void unit_test__exist_user_5(){
    struct UserCreationProcess *process = get_process(USER_CREATION_PROCESS);
    struct UserInfo test_user;
    struct UserInfo test_user2;
    struct UserInfo test_user3;
    
    memset(test_user.user_id, 0, 12);
    memset(test_user2.user_id, 0, 12);
    memset(test_user3.user_id, 0, 12);
    strcpy(test_user.user_id, "test");
    strcpy(test_user2.user_id, "a");
    strcpy(test_user3.user_id, "b");

    v_header.nextInfo = &test_user2;
    
    test_user2.prevInfo = &v_header;
    test_user2.nextInfo = &test_user3;
    
    test_user3.prevInfo = &test_user2;
    test_user3.nextInfo = &test_user;
    
    test_user.prevInfo = &test_user3;
    test_user.nextInfo = &v_tailer;
    
    v_tailer.prevInfo = &test_user3;

    unit_test_assert("NPS UTC 3000_020 USER_CREATION_PROCESS - unit_test__exist_user_5", process->_exist_user("test\0\0\0\0\0\0\0\0"));
}

// DB에 요청받은 유저id가 없는경우
static void unit_test_1() {
    struct UserCreationProcess *process = get_process(USER_CREATION_PROCESS);
    
    v_header.prevInfo = NULL;
    v_header.nextInfo = &v_tailer;
    v_tailer.prevInfo = &v_header;
    v_tailer.nextInfo = NULL;
    
   struct Packet test_user_packet = {
       .data = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0test\0\0\0\0\0\0\0\0scott\0\0\0\0\0\0\0",
       .length = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH,
       .sock = 0
   };
   
   process->trigger(&test_user_packet);
   
    unit_test_assert("NPS UTC 3000_021 USER_CREATION_PROCESS - unit_test_1", process->_exist_user("test\0\0\0\0\0\0\0\0"));
}

// DB에 요청받은 유저id가 이미 있는경우
static void unit_test_2() {
    struct UserCreationProcess *process = get_process(USER_CREATION_PROCESS);
    
    struct UserInfo test_user = {
        .user_id = "test\0\0\0\0\0\0\0\0"
    };
    
    v_header.prevInfo = NULL;
    v_header.nextInfo = &test_user;
    
    test_user.prevInfo = &v_header;
    test_user.nextInfo = &v_tailer;
    
    v_tailer.prevInfo = &test_user;
    v_tailer.nextInfo = NULL;
    
    struct Packet test_user_packet = {
       .data = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0test\0\0\0\0\0\0\0\0scott\0\0\0\0\0\0\0",
       .length = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH,
       .sock = 0
    };
   
    process->trigger(&test_user_packet);
   
    unit_test_assert("NPS UTC 3000_022 USER_CREATION_PROCESS - unit_test_2", a);
}


void test_send_packet(){
    a = 1;
}