#include <stdlib.h>
#include <string.h>
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../../databases/user_database/process.h"
#include "../../../../interfaces/network_tx_interface/process.h"

static int remove_user_id_cmp(char *, char *);
static int remove_exist_user(char *);
static void trigger();

// 프로세스 정의
static struct UserRemovalProcess user_removal_process = {
    /* Private Functions */
    ._remove_user_id_cmp = remove_user_id_cmp,
    ._remove_exist_user = remove_exist_user,
    /* Public Functions */
    .trigger = trigger
};


void init_user_removal_process() {
    printf("Initialization User Removal Process..\t\t");
    set_process(USER_REMOVAL_PROCESS, &user_removal_process);
	SET_UNIT_TEST(USER_REMOVAL_PROCESS, user_removal_process_unit_test);
	printf("\033[92m[OK]\033[0m\n");
}


static int remove_user_id_cmp(char *a, char *b) {
    int i;
    for(i=0; i<USER_ID_MAX_LENGTH; i++) 
        if(a[i] != b[i]) 
            return 1;
    return 0;
}



static int remove_exist_user(char *user_id) {
    int i;
    int flag = 0;
    struct UserDatabase *db = get_process(USER_DATABASE);
    struct UserInfo *user;

    for(user=db->header->nextInfo; user != db->tailer; user=user->nextInfo) {
        flag = 0;
        for(i=0; i<USER_ID_MAX_LENGTH && !flag; i++){ 
            if(user->user_id[i] != user_id[i]) {
                flag = 1;
                break;
            }
        }
        
        if(!flag) return 1;
    }
    
    return 0;
}

/* USER REMOVE PACKET STRUCTURE
    char req_type = 3;
    int req_id;
    short user_len;
    char user[user_len];
    short target_len;
    char target[target_len];
*/

static void trigger(struct Packet* packet) {
    struct Packet res;
    char res_data[TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH];
    memcpy(res_data, packet->data, TYPE_LENGTH + REQ_ID_LENGTH);
    
    res.data = res_data;
    res.length = TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH;
    res.sock = packet->sock;
    
    
    const int start = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH;
    if(packet->length < start + USER_ID_MAX_LENGTH) return;
    
    char *target_id = packet->data + start;
    
    // not exist id
    if(!remove_exist_user(target_id)) {
        res_data[TYPE_LENGTH + REQ_ID_LENGTH] = 1;
        ((struct NetworkTxInterface *)get_process(NETWORK_TX_INTERFACE))->send_packet(res);
        return;
    }
    
    struct UserDatabase *db = get_process(USER_DATABASE);
    struct UserInfo *tmp, *p, *n;
    
    tmp = db->header->nextInfo;
    
    while ( tmp != db->tailer )
    {
        if ( !remove_user_id_cmp(tmp->user_id, target_id) )
        {
            // 연결 삭제
            p = tmp->prevInfo;
            n = tmp->nextInfo;
            p->nextInfo = n;
            n->prevInfo = p;
            // 공간 해제
            free(tmp);
            break;
        }
        tmp = tmp->nextInfo;
    }
    
    res_data[TYPE_LENGTH + REQ_ID_LENGTH] = 0;
    // 응답 패킷 전송
    ((struct NetworkTxInterface *)get_process(NETWORK_TX_INTERFACE))->send_packet(res);
}
