#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../../databases/user_database/process.h"
#include "../../../../interfaces/network_tx_interface/process.h"


static struct UserInfo v_header;
static struct UserInfo v_tailer;

static void test_send_packet();

static struct UserDatabase test_user_database = {
    .header = &v_header,
    .tailer = &v_tailer
};

static struct NetworkTxInterface test_tx_interface = {
    .send_packet = test_send_packet
};

static void unit_test_1();
static void unit_test_2();

void user_removal_process_unit_test() {
    struct UserInfo *test_user;
    // 의존성 있는 프로세스 정의 (프로세스에서 호출하는 것들)
    set_process(USER_DATABASE, &test_user_database);
    set_process(NETWORK_TX_INTERFACE, &test_tx_interface);
    
    
    
    // 리스트 초기화 (연결 생성)
    v_header.prevInfo = NULL;
    v_header.nextInfo = &v_tailer;
    v_tailer.prevInfo = &v_header;
    v_tailer.nextInfo = NULL;
    
    // 테스트 유저 생성
    test_user = (struct UserInfo *)malloc(sizeof(struct UserInfo));
    strcpy(test_user->user_id, "bill_gates_");
    strcpy(test_user->user_pw, "bill_gates_");
    
    v_header.nextInfo = test_user;
    test_user->prevInfo = &v_header;
    test_user->nextInfo = &v_tailer;
    v_tailer.prevInfo = test_user;
    
    // 테스트
    unit_test_1();
    unit_test_2();
}

// 존재하지 않는 유저의 삭제
static void unit_test_1() {
    struct UserRemovalProcess *process = get_process(USER_REMOVAL_PROCESS);
    struct UserInfo *test_user = v_header.nextInfo;
    int res = 1;
    
    res &= !process->_remove_exist_user("steve_jobs_"); // 유저 이름이 존재하지 않는다.
    res &= process->_remove_user_id_cmp("steve_jobs_", test_user->user_id); // 테스트 유저와 비교해봐서 없는가?
    
    unit_test_assert("NPS UTC 3000_024 USER_REMOVAL_PROCESS - unit_test_1", res);
}

// 존재하는 유저를 삭제
static void unit_test_2() {
    struct UserRemovalProcess *process = get_process(USER_REMOVAL_PROCESS);
    struct Packet p;
    const int start = TYPE_LENGTH + REQ_ID_LENGTH + USER_ID_MAX_LENGTH + USER_PW_MAX_LENGTH;
    
    p.data = (char*)malloc(start + 12);
    strcpy((char*)(p.data + start), "bill_gates_");
    p.length = start+12;
    
    process->trigger(&p);
    
    unit_test_assert("NPS UTC 3000_025 USER_REMOVAL_PROCESS - unit_test_2", 
        (v_header.nextInfo == &v_tailer) &&
        (v_tailer.prevInfo == &v_header)
    );
    
    free(p.data);
}


void test_send_packet(){

}