#include <stdio.h>
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../../databases/ink_paper_state_database/process.h"

static int test_ink_value = 0;
static int test_wait_refill_ink_value = 0;
static int test_paper_value = 0;
static int test_wait_refill_paper_value = 0;

static struct InkPaperStateDatabase test_ink_paper_state_database = {
    .current_ink_value = &test_ink_value,
    .wait_refill_ink_value = &test_wait_refill_ink_value,
    .current_paper_value = &test_paper_value,
    .wait_refill_paper_value = &test_wait_refill_paper_value
};

static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();
static void unit_test_6();
static void unit_test_7();
static void unit_test_8();
static void unit_test_9();
static void unit_test_10();
static void unit_test_11();
static void unit_test_12();
static void unit_test_13();
static void unit_test_14();
static void unit_test_15();

void refill_ink_process_unit_test() {
    // 의존성 있는 프로세스 정의 (프로세스에서 호출하는 것들)
    set_process(INK_PAPER_STATE_DATABASE, &test_ink_paper_state_database);
    
    // 테스트
    unit_test_1();
    unit_test_2();
    unit_test_3();
    unit_test_4();
    unit_test_5();
    unit_test_6();
    unit_test_7();
    unit_test_8();
    unit_test_9();
    unit_test_10();
    unit_test_11();
    unit_test_12();
    unit_test_13();
    unit_test_14();
    unit_test_15();
}


//1~5 : ink_value = 0
//0 체크
static void unit_test_1() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 0;

    process->trigger();
    unit_test_assert("NPS UTC 1000_094 REFILL_INK_PROCESS - unit_test_1", test_ink_value == 0);
}

//음수 체크
static void unit_test_2() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 0;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = -50;
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_095 REFILL_INK_PROCESS - unit_test_2", test_ink_value == 0);
}

//맥스값 체크
static void unit_test_3() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 0;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 100;
    process->trigger();
    unit_test_assert("NPS UTC 1000_096 REFILL_INK_PROCESS - unit_test_3", test_ink_value == 100);
}


//오버값 체크
static void unit_test_4() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 0;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 1000;
    process->trigger();
    unit_test_assert("NPS UTC 1000_097 REFILL_INK_PROCESS - unit_test_4", test_ink_value == 100);
    
}

//기본값 체크
static void unit_test_5() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 0;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 50;
    process->trigger();
    unit_test_assert("NPS UTC 1000_098 REFILL_INK_PROCESS - unit_test_5", test_ink_value == 50);
}






//6~10 : ink_value = 1500
//0 체크
static void unit_test_6() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 1500;
    test_wait_refill_ink_value = 0;
    
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_099 REFILL_INK_PROCESS - unit_test_6", test_ink_value == 1500);
}

//음수 체크
static void unit_test_7() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 1500;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = -50;
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_100 REFILL_INK_PROCESS - unit_test_7", test_ink_value == 1500);
}

//맥스값 체크
static void unit_test_8() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 1500;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 100;
    process->trigger();
    unit_test_assert("NPS UTC 1000_101 REFILL_INK_PROCESS - unit_test_8", test_ink_value == 1600);
}


//오버값 체크
static void unit_test_9() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 1500;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 1000;
    process->trigger();
    unit_test_assert("NPS UTC 1000_102 REFILL_INK_PROCESS - unit_test_9", test_ink_value == 1600);
    
}

//기본값 체크
static void unit_test_10() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = 1500;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 50;
    process->trigger();
    unit_test_assert("NPS UTC 1000_103 REFILL_INK_PROCESS - unit_test_10", test_ink_value == 1550);
}





//11 ~ 15 : ink_value = INK_MAX_VALUE
//0 체크
static void unit_test_11() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = INK_MAX_VALUE;
    test_wait_refill_ink_value = 0;
    
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_104 REFILL_INK_PROCESS - unit_test_11", test_ink_value == INK_MAX_VALUE);
}

//음수 체크
static void unit_test_12() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = INK_MAX_VALUE;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = -50;
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_105 REFILL_INK_PROCESS - unit_test_12", test_ink_value == INK_MAX_VALUE);
}

//맥스값 체크
static void unit_test_13() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = INK_MAX_VALUE;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 100;
    process->trigger();
    unit_test_assert("NPS UTC 1000_106 REFILL_INK_PROCESS - unit_test_13", test_ink_value == INK_MAX_VALUE);
}


//오버값 체크
static void unit_test_14() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = INK_MAX_VALUE;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 1000;
    process->trigger();
    unit_test_assert("NPS UTC 1000_107 REFILL_INK_PROCESS - unit_test_14", test_ink_value == INK_MAX_VALUE);
    
}

//기본값 체크
static void unit_test_15() {
    struct RefillInkProcess *process = get_process(REFILL_INK_PROCESS);
    test_ink_value = INK_MAX_VALUE;
    test_wait_refill_ink_value = 0;
    
    test_wait_refill_ink_value = 50;
    process->trigger();
    unit_test_assert("NPS UTC 1000_108 REFILL_INK_PROCESS - unit_test_15", test_ink_value == INK_MAX_VALUE);
}
