#include <stdio.h>
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../../databases/ink_paper_state_database/process.h"

static int test_ink_value = 0;
static int test_wait_refill_ink_value = 0;
static int test_paper_value = 0;
static int test_wait_refill_paper_value = 0;

static struct InkPaperStateDatabase test_ink_paper_state_database = {
    .current_ink_value = &test_ink_value,
    .wait_refill_ink_value = &test_wait_refill_ink_value,
    .current_paper_value = &test_paper_value,
    .wait_refill_paper_value = &test_wait_refill_paper_value
};

static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();
static void unit_test_6();
static void unit_test_7();
static void unit_test_8();
static void unit_test_9();
static void unit_test_10();
static void unit_test_11();
static void unit_test_12();
static void unit_test_13();
static void unit_test_14();
static void unit_test_15();

void refill_paper_process_unit_test() {
    // 의존성 있는 프로세스 정의 (프로세스에서 호출하는 것들)
    set_process(INK_PAPER_STATE_DATABASE, &test_ink_paper_state_database);
    
    // 테스트
    unit_test_1();
    unit_test_2();
    unit_test_3();
    unit_test_4();
    unit_test_5();
    unit_test_6();
    unit_test_7();
    unit_test_8();
    unit_test_9();
    unit_test_10();
    unit_test_11();
    unit_test_12();
    unit_test_13();
    unit_test_14();
    unit_test_15();
}


//1~5 : paper_value = 0
//0 체크
static void unit_test_1() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 0;

    process->trigger();
    unit_test_assert("NPS UTC 1000_109 REFILL_PAPER_PROCESS - unit_test_1", test_paper_value == 0);
}

//음수 체크
static void unit_test_2() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 0;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = -5;
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_110 REFILL_PAPER_PROCESS - unit_test_2", test_paper_value == 0);
}

//맥스값 체크
static void unit_test_3() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 0;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 10;
    process->trigger();
    unit_test_assert("NPS UTC 1000_111 REFILL_PAPER_PROCESS - unit_test_3", test_paper_value == 10);
}


//오버값 체크
static void unit_test_4() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 0;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 1000;
    process->trigger();
    unit_test_assert("NPS UTC 1000_112 REFILL_PAPER_PROCESS - unit_test_4", test_paper_value == 10);
    
}

//기본값 체크
static void unit_test_5() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 0;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 5;
    process->trigger();
    unit_test_assert("NPS UTC 1000_113 REFILL_PAPER_PROCESS - unit_test_5", test_paper_value == 5);
}






//6~10 : paper_value = 50
//0 체크
static void unit_test_6() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 50;
    test_wait_refill_paper_value = 0;
    
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_114 REFILL_PAPER_PROCESS - unit_test_6", test_paper_value == 50);
}

//음수 체크
static void unit_test_7() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 50;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = -50;
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_115 REFILL_PAPER_PROCESS - unit_test_7", test_paper_value == 50);
}

//맥스값 체크
static void unit_test_8() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 50;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 10;
    process->trigger();
    unit_test_assert("NPS UTC 1000_116 REFILL_PAPER_PROCESS - unit_test_8", test_paper_value == 60);
}


//오버값 체크
static void unit_test_9() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 50;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 10;
    process->trigger();
    unit_test_assert("NPS UTC 1000_117 REFILL_PAPER_PROCESS - unit_test_9", test_paper_value == 60);
    
}

//기본값 체크
static void unit_test_10() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = 50;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 5;
    process->trigger();
    unit_test_assert("NPS UTC 1000_118 REFILL_PAPER_PROCESS - unit_test_10", test_paper_value == 55);
}





//11 ~ 15 : paper_value = PAPER_MAX_VALUE
//0 체크
static void unit_test_11() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = PAPER_MAX_VALUE;
    test_wait_refill_paper_value = 0;
    
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_119 REFILL_PAPER_PROCESS - unit_test_11", test_paper_value == PAPER_MAX_VALUE);
}

//음수 체크
static void unit_test_12() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = PAPER_MAX_VALUE;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = -50;
    
    process->trigger();
    unit_test_assert("NPS UTC 1000_120 REFILL_PAPER_PROCESS - unit_test_12", test_paper_value == PAPER_MAX_VALUE);
}

//맥스값 체크
static void unit_test_13() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = PAPER_MAX_VALUE;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 100;
    process->trigger();
    unit_test_assert("NPS UTC 1000_121 REFILL_PAPER_PROCESS - unit_test_13", test_paper_value == PAPER_MAX_VALUE);
}


//오버값 체크
static void unit_test_14() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = PAPER_MAX_VALUE;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 1000;
    process->trigger();
    unit_test_assert("NPS UTC 1000_122 REFILL_PAPER_PROCESS - unit_test_14", test_paper_value == PAPER_MAX_VALUE);
    
}

//기본값 체크
static void unit_test_15() {
    struct RefillPaperProcess *process = get_process(REFILL_PAPER_PROCESS);
    test_paper_value = PAPER_MAX_VALUE;
    test_wait_refill_paper_value = 0;
    
    test_wait_refill_paper_value = 50;
    process->trigger();
    unit_test_assert("NPS UTC 1000_123 REFILL_PAPER_PROCESS - unit_test_15", test_paper_value == PAPER_MAX_VALUE);
}
