#include "../../../../system.h"
#include "../../../../process.h"
#include "../../../../unittest.h"
#include "../../../../databases/ink_paper_state_database/process.h"
#include "../../../../interfaces/virtual_refill_interface/process.h"
#include "../refill_ink_process/process.h"
#include "../refill_paper_process/process.h"
#include "../reset_wait_refill_ink_process/process.h"
#include "../reset_wait_refill_paper_process/process.h"
#include "../wait_refill_ink_addition_process/process.h"
#include "../wait_refill_paper_addition_process/process.h"
#include "process.h"
#include "test.h"


static void tick();


static struct VirtualRefillController virtual_refill_controller = {
    .tick = tick
};

static int tick_count;
static void (*current_state)();

static void idle();
static void wait_refill_ink();
static void refill_ink();
static void wait_refill_paper();
static void refill_paper();

void init_virtual_refill_controller() {
    printf("Initialization Virtual Refill Controller...\t\t");
    set_process(VIRTUAL_REFILL_CONTROLLER, &virtual_refill_controller);
    SET_UNIT_TEST(VIRTUAL_REFILL_CONTROLLER, virtual_refill_controller_unit_test);
    add_tick_listener(virtual_refill_controller.tick);
    printf("\033[92m[OK]\033[0m\n");
    
    current_state = idle;
    tick_count = 0;
}


static void tick()
{
    current_state();
}

static void idle() {
    struct RequestRefillInfo request = ((struct VirtualRefillInterface *)get_process(VIRTUAL_REFILL_INTERFACE))->get_request_info();

    // Tick [RequestRefillInfo.type == REFILL_TYPE_INK && RequestRefillInfo.value > 0]
    if(request.type == REFILL_TYPE_INK && request.value > 0) {
        ((struct WaitRefillInkAdditionProcess *)get_process(WAIT_REFILL_INK_ADDITION_PROCESS))->trigger(request.value);
        current_state = wait_refill_ink;
    }
    
    // Tick [RequestRefillInfo.type == REFILL_TYPE_INK && RequestRefillInfo.value > 0]
    if(request.type == REFILL_TYPE_PAPER && request.value > 0) {
        ((struct WaitRefillPaperAdditionProcess *)get_process(WAIT_REFILL_PAPER_ADDITION_PROCESS))->trigger(request.value);
        current_state = wait_refill_paper;
    }
}

static void wait_refill_ink() {
    struct InkPaperStateDatabase *db = get_process(INK_PAPER_STATE_DATABASE);
    
    // Tick [(++tick_count) % 20 == 0 && wait_refill_ink_value > 0 && current_ink_value < INK_MAX_VALUE ]
    if((++tick_count) % 10 == 0 && *(db->wait_refill_ink_value) > 0 && *(db->current_ink_value) < INK_MAX_VALUE) {
        ((struct RefillInkProcess *)get_process(REFILL_INK_PROCESS))->trigger();
        current_state = refill_ink;
    }
    
    // Tick [wait_refill_ink_value < 1 || current_ink_value >= INK_MAX_VALUE ]
    if(*(db->wait_refill_ink_value) < 1 || *(db->current_ink_value) >= INK_MAX_VALUE) {
        ((struct ResetWaitRefillInkProcess *)get_process(RESET_WAIT_REFILL_INK_PROCESS))->trigger();
        tick_count = 0;
        current_state = idle;
    }
}

static void refill_ink() {
    current_state = wait_refill_ink;
}

static void wait_refill_paper() {
    struct InkPaperStateDatabase *db = get_process(INK_PAPER_STATE_DATABASE);
    
    // Tick [(++tick_count) % 20 == 0 && wait_refill_paper_value > 0 && current_paper_value < PAPER_MAX_VALUE ]
    if((++tick_count) % 10 == 0 && *(db->wait_refill_paper_value) > 0 && *(db->current_paper_value) < PAPER_MAX_VALUE) {
        ((struct RefillPaperProcess *)get_process(REFILL_PAPER_PROCESS))->trigger();
        current_state = refill_paper;
    }
    
    // Tick [wait_refill_paper_value < 1 || current_paper_value >= INK_MAX_VALUE ]
    if(*(db->wait_refill_paper_value) < 1 || *(db->current_paper_value) >= INK_MAX_VALUE) {
        ((struct ResetWaitRefillPaperProcess *)get_process(RESET_WAIT_REFILL_PAPER_PROCESS))->trigger();
        tick_count = 0;
        current_state = idle;
    }
}

static void refill_paper() {
    current_state = wait_refill_paper;
}
