#include <arpa/inet.h>
#include <sys/socket.h>
#include "process.h"
#include "test.h"
#include "../../../../process.h"
#include "../../../../system.h"
#include "../../../../unittest.h"
#include "../../../../databases/ink_paper_state_database/process.h"
#include "../../../../interfaces/virtual_refill_interface/process.h"

static int test_ink_value = 0;
static int test_wait_refill_ink_value = 0;
static int test_paper_value = 0;
static int test_wait_refill_paper_value = 0;

static struct RequestRefillInfo test_request_refill_info;

static struct RequestRefillInfo test_get_request_info() {
    return test_request_refill_info;
}

static struct InkPaperStateDatabase test_ink_paper_state_database = {
    .current_ink_value = &test_ink_value,
    .wait_refill_ink_value = &test_wait_refill_ink_value,
    .current_paper_value = &test_paper_value,
    .wait_refill_paper_value = &test_wait_refill_paper_value
};

static struct VirtualRefillInterface test_virtual_interface = {
    .get_request_info = test_get_request_info
};

static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();
static void unit_test_6();

void virtual_refill_controller_unit_test() {
    // 의존성 있는 프로세스 정의 (프로세스에서 호출하는 것들)
    set_process(INK_PAPER_STATE_DATABASE, &test_ink_paper_state_database);
    set_process(VIRTUAL_REFILL_INTERFACE, &test_virtual_interface);

    // 테스트
    unit_test_1();
    unit_test_2();
    unit_test_3();
    unit_test_4();
    unit_test_5();
    unit_test_6();
}

/* Ink */

//IDLE => Wait_Refill_Ink 
static void unit_test_1() {
    struct InkPaperStateDatabase *ipsdb = get_process(INK_PAPER_STATE_DATABASE);
    struct VirtualRefillController *controller = get_process(VIRTUAL_REFILL_CONTROLLER);
    
    test_request_refill_info.type = 0;
    test_request_refill_info.value = 2570;
    
    controller->tick();
    
    unit_test_assert("NPS UTC 1000_125 VIRTUAL_REFILL_CONTROLLER - unit_test_1", *(ipsdb->wait_refill_ink_value) == 2570);
}


//Wait_Refill_Ink => Reill_Ink => Wait_refill_Ink
static void unit_test_2(){
    int i;
    struct InkPaperStateDatabase *ipsdb = get_process(INK_PAPER_STATE_DATABASE);
    struct VirtualRefillController *controller = get_process(VIRTUAL_REFILL_CONTROLLER);
    
    for(i = 0; i < 21; i++){
        controller->tick();
    }
    
    unit_test_assert("NPS UTC 1000_126 VIRTUAL_REFILL_CONTROLLER - unit_test_2", *(ipsdb->wait_refill_ink_value) == 2370 && *(ipsdb->current_ink_value) == 200);
}

//Wait_Refill_Ink => ... => IDLE
static void unit_test_3(){
    struct InkPaperStateDatabase *ipsdb = get_process(INK_PAPER_STATE_DATABASE);
    struct VirtualRefillController *controller = get_process(VIRTUAL_REFILL_CONTROLLER);
    
    while(*(ipsdb->wait_refill_ink_value)){
        controller->tick();
    }
    
    unit_test_assert("NPS UTC 1000_127 VIRTUAL_REFILL_CONTROLLER - unit_test_3", *(ipsdb->wait_refill_ink_value) == 0 && *(ipsdb->current_ink_value) == 2570);
}


/* Paper */

//IDLE => Wait_Refill_Paper 
static void unit_test_4() {
    struct InkPaperStateDatabase *ipsdb = get_process(INK_PAPER_STATE_DATABASE);
    struct VirtualRefillController *controller = get_process(VIRTUAL_REFILL_CONTROLLER);
    
    test_request_refill_info.type = 1;
    test_request_refill_info.value = 60;
    
    controller->tick();
    
    unit_test_assert("NPS UTC 1000_128 VIRTUAL_REFILL_CONTROLLER - unit_test_4", *(ipsdb->wait_refill_paper_value) == 60);
}


//Wait_Refill_Paper => Reill_Paper => Wait_refill_Paper
static void unit_test_5(){
    int i;
    struct InkPaperStateDatabase *ipsdb = get_process(INK_PAPER_STATE_DATABASE);
    struct VirtualRefillController *controller = get_process(VIRTUAL_REFILL_CONTROLLER);
    
    for(i = 0; i < 21; i++){
        controller->tick();
    }
    
    unit_test_assert("NPS UTC 1000_129 VIRTUAL_REFILL_CONTROLLER - unit_test_5", *(ipsdb->wait_refill_paper_value) == 40 && *(ipsdb->current_paper_value) == 20);
}

//Wait_Refill_Paper => ... => IDLE
static void unit_test_6(){
    struct InkPaperStateDatabase *ipsdb = get_process(INK_PAPER_STATE_DATABASE);
    struct VirtualRefillController *controller = get_process(VIRTUAL_REFILL_CONTROLLER);
    
    while(*(ipsdb->wait_refill_paper_value)){
        controller->tick();
    }
    
    unit_test_assert("NPS UTC 1000_130 VIRTUAL_REFILL_CONTROLLER - unit_test_6", *(ipsdb->wait_refill_paper_value) == 0 && *(ipsdb->current_paper_value) == 60);
}
