#include "../../../process.h"
#include "../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../databases/ink_paper_state_database/process.h"
#include "../../../databases/job_queue/process.h"
#include "../../../interfaces/save_print_result/process.h"
#include "../job_queue_pop/process.h"
#include "../next_page/process.h"
#include "../decrease_amount_of_ink_paper/process.h"

static void tick();

static void idle();
static void job_ready();
static void end_job();
static void system_wait();
static void printing();

static void (*current_state)();

// 프로세스 정의
static struct PrintingController printing_controller = {
    .tick = tick
};

static int tick_count;


void init_printing_controller() {
    printf("Initialization Printing Controller...\t\t");
    set_process(PRINTING_CONTROLLER, &printing_controller);
	SET_UNIT_TEST(PRINTING_CONTROLLER, printing_controller_unit_test);
	add_tick_listener(printing_controller.tick);
	printf("\033[92m[OK]\033[0m\n");
	
	current_state = idle;
	tick_count = 0;
}

static void tick() {
    current_state();
}

static void idle() {
    struct JobQueue *job_queue = get_process(JOB_QUEUE);
    
    if(job_queue->jobs_info[0].job_id != -1) {
        current_state = job_ready;
    }
}

static int get_page_require_ink(char *buf, int len, int page) {
    int i;
    int line_count = 0;
    int require_ink = 0;
    int line_text_length = 0;
    int sq = 0;
    
    for(i=0; i<len && line_count/10 <= page; i++) {
        if(buf[i] != '\n') {
            if(line_text_length < 30) {
                line_text_length += 1;
                if(buf[i] == ' ') sq += 1;
            }
        } else {
            if(line_count/10 == page) {
                require_ink += line_text_length - sq;
            }
            line_count++;
            line_text_length = 0;
            sq = 0;
        }
    }
    
    return require_ink;
}

static void job_ready() {
    struct JobQueue *job_queue = get_process(JOB_QUEUE);
    struct InkPaperStateDatabase *ink_paper_db = get_process(INK_PAPER_STATE_DATABASE);

    // Tick [ !Next Page Exist? || Job State == Removing]
    if(job_queue->jobs_info[0].current_page >= job_queue->jobs_info[0].total_page || job_queue->jobs_info[0].state == REMOVING) {
        tick_count = 0;
        current_state = end_job;
        return;
    }
    
    // Tick [wait_refill_ink_value > 0 || wait_refill_paper_value > 0]
    if(*(ink_paper_db->wait_refill_ink_value) > 0 || *(ink_paper_db->wait_refill_paper_value) > 0) {
        tick_count = 0;
        current_state = system_wait;
        return;
    }
    
    // Tick [Current Page Exist ?]
    if((++tick_count) % 20 == 0 && *(ink_paper_db->wait_refill_ink_value) < 1 && *(ink_paper_db->wait_refill_paper_value) < 1 && job_queue->jobs_info[0].current_page < job_queue->jobs_info[0].total_page && job_queue->jobs_info[0].state != REMOVING) {
        int require_ink = get_page_require_ink(job_queue->jobs_info[0].buffer, job_queue->jobs_info[0].file_size, job_queue->jobs_info[0].current_page);
        
        ((struct DecreaseAmountOfInkPaper *)get_process(DECREASE_AMOUNT_OF_INK_PAPER))->trigger(require_ink);
        ((struct SavePrintResult *)get_process(SAVE_PRINT_RESULT))->trigger();
        ((struct NextPage *)get_process(NEXT_PAGE))->trigger();
        
        current_state = printing;
        return;
    }
}

static void end_job() {
    ((struct JobQueuePop *)get_process(JOB_QUEUE_POP))->trigger();
    current_state = idle;
}

static void system_wait() {
    struct InkPaperStateDatabase *ink_paper_db = get_process(INK_PAPER_STATE_DATABASE);
    
    // Tick [wait_refill_ink_value < 1 && wait_refill_paper_value < 1]
    if(*(ink_paper_db->wait_refill_ink_value) < 1 && *(ink_paper_db->wait_refill_paper_value) < 1) {
        current_state = job_ready;
    }
}

static void printing() {
    current_state = job_ready;
}
