#include <stdio.h>
#include "../../../process.h"
#include "../../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../../databases/job_queue/process.h"
#include "../../../databases/ink_paper_state_database/process.h"
#include "../decrease_amount_of_ink_paper/process.h"
#include "../../../interfaces/save_print_result/process.h"
#include "../next_page/process.h"
#include "../job_queue_pop/process.h"

static int test_ink_value = INK_MAX_VALUE;
static int test_wait_refill_ink_value = 0;
static int test_paper_value = PAPER_MAX_VALUE;
static int test_wait_refill_paper_value = 0;


static int flag = 0;

static const int flag_decrease_amount = 1;
static const int flag_save_print_result = 2;
static const int flag_next_page = 4;
static const int flag_job_queue_pop = 8;


static void test_trigger1(){
    flag |= flag_decrease_amount;
}
static void test_trigger2(){
    flag |= flag_save_print_result;
}
static void test_trigger3(){
    flag |= flag_next_page;
}
static void test_trigger4(){
    flag |= flag_job_queue_pop;
}

static struct DecreaseAmountOfInkPaper test_decrease_amount_of_ink_paper = {
    .trigger = test_trigger1
};
static struct SavePrintResult test_save_print_result = {
    .trigger = test_trigger2
};
static struct NextPage test_next_page = {
    .trigger = test_trigger3
};
static struct JobQueuePop test_job_queue_pop = {
    .trigger = test_trigger4
};


static struct InkPaperStateDatabase test_ink_paper_state_database = {
    .current_ink_value = &test_ink_value,
    .wait_refill_ink_value = &test_wait_refill_ink_value,
    .current_paper_value = &test_paper_value,
    .wait_refill_paper_value = &test_wait_refill_paper_value
};


static struct JobInfo test_jobs_info[JOB_MAX_SIZE];

static struct JobQueue test_job_queue = {
    .jobs_info = test_jobs_info
};

static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();

void printing_controller_unit_test() {
    int i;
    // 의존성 있는 프로세스 정의 (프로세스에서 호출하는 것들)
    set_process(JOB_QUEUE, &test_job_queue);
    set_process(INK_PAPER_STATE_DATABASE, &test_ink_paper_state_database);
    set_process(DECREASE_AMOUNT_OF_INK_PAPER, &test_decrease_amount_of_ink_paper);
    set_process(SAVE_PRINT_RESULT, &test_save_print_result);
    set_process(NEXT_PAGE, &test_next_page);
    set_process(JOB_QUEUE_POP, &test_job_queue_pop);
    
    
    for ( i = 0 ; i < JOB_MAX_SIZE ; i++ )
    {
        test_jobs_info[i].fd = NULL;
        test_jobs_info[i].job_id = i+1;
        memset(test_jobs_info[i].user_id, 0, 12);
        sprintf(test_jobs_info[i].user_id, "user%d", i);
        test_jobs_info[i].current_page = rand() % 5;
        test_jobs_info[i].total_page = i+5;
        test_jobs_info[i].state = 0;
        test_jobs_info[i].buffer = (char*)malloc(32);
        memset(test_jobs_info[i].buffer, 65+i, 31);
        test_jobs_info[i].buffer[31] = 0;
    }
    
    // 테스트
    unit_test_1();
    unit_test_2();
    unit_test_3();
    unit_test_4();
    unit_test_5();
}

//IDLE there is no job
//don't call any function ?? 
static void unit_test_1() {
    struct PrintingController *controller = get_process(PRINTING_CONTROLLER);
    int i;
    test_jobs_info[0].job_id = -1;
    
    controller->tick();
    for(i = 0; i < 20; i++) controller->tick();
    controller->tick();
    
    unit_test_assert("NPS UTC 5000_014 PRINTING_CONTROLLER - unit_test_1", flag == 0);
    
    test_jobs_info[0].job_id = 1;
    flag = 0;
}


//IDLE -> job_ready -> Printing -> job_ready
//call Decrease amount, Save Print result, Next Page ??
static void unit_test_2() {
    struct PrintingController *controller = get_process(PRINTING_CONTROLLER);
    int i;

    controller->tick();
    for(i = 0; i < 20; i++) controller->tick();
    controller->tick();
    
    unit_test_assert("NPS UTC 5000_015 PRINTING_CONTROLLER - unit_test_2", flag == 7);
    
    flag = 0;
}

//job_ready -> System_Wait -> job_ready
//!(call Decrease amount, Save 
static void unit_test_3() {
    struct PrintingController *controller = get_process(PRINTING_CONTROLLER);
    test_wait_refill_ink_value = 1;
    
    controller->tick();
    
    unit_test_assert("NPS UTC 5000_016 PRINTING_CONTROLLER - unit_test_3", flag == 0);
    
    test_wait_refill_ink_value = 0;
    controller->tick();
    
    flag = 0;
}

//job_ready -> end_job -> IDLE (currentPage = totalPage)
//call job_queue_pop ??
static void unit_test_4() {
    struct PrintingController *controller = get_process(PRINTING_CONTROLLER);
    test_jobs_info[0].current_page = test_jobs_info[0].total_page;
    
    controller->tick();
    controller->tick();
    
    unit_test_assert("NPS UTC 5000_017 PRINTING_CONTROLLER - unit_test_4", flag == 8);
    
    flag = 0;
}

//IDLE -> job_ready -> end_job -> IDLE (removing)
//call job_queue_pop ??
static void unit_test_5() {
    struct PrintingController *controller = get_process(PRINTING_CONTROLLER);
    
    controller->tick();
    test_jobs_info[0].state = REMOVING;
    
    controller->tick();
    controller->tick();
    
    unit_test_assert("NPS UTC 5000_017 PRINTING_CONTROLLER - unit_test_5", flag == 8);
    
    flag = 0;
}