#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "process.h"
#include "test.h"
#include "../../../process.h"
#include "../../../system.h"
#include "../../../unittest.h"
#include "../../../databases/ink_paper_state_database/process.h"
#include "../../../databases/user_database/process.h"
#include "../../../databases/job_queue/process.h"
#include "../../../interfaces/lcd_interface/process.h"
#include "../../../interfaces/network_tx_interface/process.h"

static void _transform(char *buf, int length);
static void trigger();

static struct SendStatusProcess send_status_process = {
    // Private Function
    ._transform = _transform,
    // Public Function
    .trigger = trigger
};

void init_send_status_process() {
    printf("Initialization Send Status Process...\t\t");
    set_process(SEND_STATUS_PROCESS, &send_status_process);
    SET_UNIT_TEST(SEND_STATUS_PROCESS, send_status_process_unit_test);
    printf("\033[92m[OK]\033[0m\n");
}

static void _transform(char *buf, int length) {
    char temp[1024];
    char time_str[256];
    char *state_char[] = {
        "idle",
        "printing",
        "system wait(refill)",
        "removing"
    };
    
    int i;
    time_t     tm_time;
    struct tm* st_time;
    
    struct InkPaperStateDatabase *ipsdb = get_process(INK_PAPER_STATE_DATABASE);
    struct JobQueue *jq = get_process(JOB_QUEUE);
    
    int current_ink_value = *(ipsdb->current_ink_value);
    int current_paper_value = *(ipsdb->current_paper_value);
    int wait_refill_ink_value = *(ipsdb->wait_refill_ink_value);
    int wait_refill_paper_value = *(ipsdb->wait_refill_paper_value);

    time(&tm_time);
    #ifndef UNIT_TEST //꼼수 ㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋㅋ 몰라 젠장
    st_time = localtime(&tm_time);
    strftime(time_str, 256, "%Y-%m-%d %H:%M:%S", st_time);
    #else
    time_str[0] = 0;
    #endif
    
    // 문서참고 문서에 뭐뭐띄워야 하는지 나와있음 (요구사항 문서)
    // 그리고 0: 인쇄 혹은 대기임 1:삭제중
    // 인쇄, 대기 판별법은
    // 일단 첫번째 잡이아니면 무조건 대기고
    // 첫번째 잡이라고 하더라도 리필중인 잔량이 있으면 대기임
    
    // state 0 idle, 1 printing, 2: refill wait, 3 removing
    int state = (*(ipsdb->wait_refill_ink_value) > 0 || *(ipsdb->wait_refill_paper_value) > 0) ? 2 : jq->jobs_info[0].job_id == -1 ? 0 : jq->jobs_info[0].state == REMOVING ? 3 : 1;

    sprintf(buf,
        "Network Printer System\n" \
        "Time: %s\n" \
        "State: %s\n" \
        "[Ink]\t[Paper]\n" \
        "%d\t%d\n" \
        "[Job Queue]\n" \ 
        "ID\tUser\tState\tPage\n",
        time_str,
        state_char[state], 
        current_ink_value, current_paper_value);

    for(i=0; i<JOB_MAX_SIZE && jq->jobs_info[i].job_id != -1; i++) {
        sprintf(temp, "%d\t%s\t%s\t%d/%d\n",
            jq->jobs_info[i].job_id,
            jq->jobs_info[i].user_id,
            i != 0 ? "Wait" : state == 2 ? "Wait" : "Printing",
            jq->jobs_info[i].current_page, jq->jobs_info[i].total_page
        );
        strcat(buf, temp);
    }
}

static void trigger() {
    char msg[1024];
    char packet_msg[1024 + TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH + 4];
    
    struct NetworkTxInterface *tx = get_process(NETWORK_TX_INTERFACE);
    struct LCDInterface *lcd = get_process(LCD_INTERFACE);
    struct Packet packet;
    
    int str_len;
    
    _transform(msg, 1024);
    
    str_len = strlen(msg);
    
    packet_msg[0] = -1;
    packet_msg[1] = 0;
    packet_msg[2] = 0;
    packet_msg[3] = 0;
    packet_msg[4] = 0;
    packet_msg[5] = 0;
    packet_msg[6] = (str_len >> 24) & 0xff;
    packet_msg[7] = (str_len >> 16) & 0xff;
    packet_msg[8] = (str_len >> 8) & 0xff;
    packet_msg[9] = str_len & 0xff;
    
    memcpy(packet_msg + 10, msg, str_len * sizeof(char));
    
    packet.sock = -1; // 브로드 캐스트
    packet.data = packet_msg;
    packet.length = TYPE_LENGTH + REQ_ID_LENGTH + RES_TYPE_LENGTH + 4 + (str_len * sizeof(char));
    
    // 1. Network Tx Interface로 패킷 전송
    // 2. LCD Interface로 데이터 전송
    tx->send_packet(packet);
    lcd->print(msg);
}
