#include <string.h>
#include <stdlib.h>
#include "process.h"
#include "test.h"
#include "../../../process.h"
#include "../../../system.h"
#include "../../../unittest.h"
#include "../../../databases/ink_paper_state_database/process.h"
#include "../../../databases/job_queue/process.h"

static struct JobInfo test_jobs_info[JOB_MAX_SIZE];
static struct JobQueue test_job_queue = {
    .jobs_info = test_jobs_info    
};

static test_ink_value = 150;
static test_paper_value = 10;
static test_wait_refill_paper_value = 0;
static test_wait_refill_ink_value = 0;

static struct InkPaperStateDatabase test_ink_paper_state_database = {
    .current_ink_value = &test_ink_value,
    .current_paper_value = &test_paper_value,
    .wait_refill_ink_value = &test_wait_refill_ink_value,
    .wait_refill_paper_value = &test_wait_refill_paper_value
};

static void unit_test_1();
static void unit_test_2();
static void unit_test_3();
static void unit_test_4();
static void unit_test_5();

void send_status_process_unit_test() {
    int i;
    // 의존 프로세스 로드
    set_process(INK_PAPER_STATE_DATABASE, &test_ink_paper_state_database);
    set_process(JOB_QUEUE, &test_job_queue);
    
    srand(time(NULL));
    // Job Queue 초기화
    for ( i = 0 ; i < JOB_MAX_SIZE ; i++ )
    {
        test_jobs_info[i].job_id = i+1;
        memset(test_jobs_info[i].user_id, 0, 12);
        sprintf(test_jobs_info[i].user_id, "user%d", i);
        test_jobs_info[i].current_page = rand() % 6;
        test_jobs_info[i].total_page = i+5;
    }
    
    test_jobs_info[0].state = 0;
    
    unit_test_1();
    
    test_wait_refill_ink_value = 1;
    
    unit_test_2();
    
    test_wait_refill_ink_value = 0;
    test_jobs_info[0].state = 1;
    
    unit_test_3();
    
    test_ink_value -= 10;
    
    unit_test_4();
    
    test_paper_value -= 5;
    
    unit_test_5();
    
}

// current state 가 제대로 표시되는지 확인! (1, 2, 3)
static void unit_test_1() {
    char buf[1024];
    
    struct SendStatusProcess *process = get_process(SEND_STATUS_PROCESS);

    process->_transform(buf, 1024);
    unit_test_assert("NPS UTC 4000_000 SEND_STATUS_PROCESS - unit_test_1", 
        (strstr(buf, "\nState: printing\n") != NULL)
    );
}

static void unit_test_2() {
    char buf[1024];
    
    struct SendStatusProcess *process = get_process(SEND_STATUS_PROCESS);
    process->_transform(buf, 1024);
    
    unit_test_assert("NPS UTC 4000_001 SEND_STATUS_PROCESS - unit_test_2", 
        (strstr(buf, "\nState: system wait(refill)\n") != NULL)
    );
}

static void unit_test_3() {
    char buf[1024];
    
    struct SendStatusProcess *process = get_process(SEND_STATUS_PROCESS);
    
    process->_transform(buf, 1024);
    unit_test_assert("NPS UTC 4000_002 SEND_STATUS_PROCESS - unit_test_3",
        (strstr(buf, "\nState: removing\n") != NULL)
    );
}

// 잉크와 종이 값이 제대로 변경되는가?
// 잉크 값 조절
static void unit_test_4() {
    char buf[1024];
    
    struct SendStatusProcess *process = get_process(SEND_STATUS_PROCESS);
    process->_transform(buf, 1024);
    unit_test_assert("NPS UTC 4000_003 SEND_STATUS_PROCESS - unit_test_4",
        (strstr(buf, "\n[Ink]\t") != NULL) &&
        (strstr(buf, "140") != NULL)
    );
}

// 종이 값 조절
static void unit_test_5() {
    char buf[1024];
    
    struct SendStatusProcess *process = get_process(SEND_STATUS_PROCESS);
    process->_transform(buf, 1024);
    unit_test_assert("NPS UTC 4000_004 SEND_STATUS_PROCESS - unit_test_5",
        (strstr(buf, "[Paper]\n") != NULL) &&
        (strstr(buf, "5") != NULL)
    );
}